
//===============================================================
// Lawson Labs, Inc.
// 3217 Phoenixville Pike
// Malvern, PA  19355
// 800 321-5355 or 610-725-8800
//-----------------------------
// Programmer: Tim Van Dusen
// Last modified: 3-12-18
// Name: VC5_APP.cpp
// Uses LL_USB2K.DLL
//===============================================================
  
#include <windows.h>
#include <stdlib.h>
#include <wtypes.h>
#include <conio.h>
#include <stdio.h>
#include <time.h>
#include <math.h>

#include "vc5_app.h"
#include "resource.h"
//NOTE: Remove the following line to see build "warnings".
#pragma warning( disable : 4417 4700 4996 )



//****************************************************************\
//================================================================*\
//==== MAIN FUNCTION ==============================================*>
//================================================================*/
/*****************************************************************/
#include <string>
#include <iostream>


// Get the horizontal and vertical screen sizes in pixel
void GetDesktopResolution(int* horizontal, int* vertical)
{
   RECT desktop;
   // Get a handle to the desktop window
   const HWND hDesktop = GetDesktopWindow();
   // Get the size of screen to the variable desktop
   GetWindowRect(hDesktop, &desktop);
   // The top left corner will have coordinates (0,0)
   // and the bottom right corner will have coordinates
   // (horizontal, vertical)
   *horizontal = desktop.right;
   *vertical = desktop.bottom;
}

int scHeight;
int __cdecl main()
{
	int horizontal = 0;
	int vertical = 0;
	GetDesktopResolution(&horizontal, &vertical);
	scHeight = vertical / 20;
	int scWidth = SCRN_WIDTH;
	char buffM[32];
	sprintf(buffM, "mode %u, %u", scWidth, scHeight);
	system(buffM);
	hAppWindow = GetForegroundWindow();

	// IN-HOUSE TESTING ONLY MakeDialog();

	if(!SetupConsole()){
		MessageBox(0, "SetupConsole() failed.  EXITING APPLICATION", "-- error", MB_OK);
		return 1;}

	if(!LoadLibraries()){
		MessageBox(hAppWindow, "LoadLibraries() failed.  EXITING APPLICATION", "-- error", MB_OK);
		return 1;}

	//5-25-18 disable the "x" close button so app can stop scan if scanning before exit
	EnableMenuItem(GetSystemMenu( hAppWindow, FALSE), SC_CLOSE, MF_BYCOMMAND | MF_GRAYED );

	//5-25-18 removes all menu items, except "properties", so app cannot be closed by menu, preventing board left scanning, if scanning
	// note that menu item index always 0 since as one is deleted, the next moves into 0 position.
	//Could also use something like: DeleteMenu(hMenu, SC_CLOSE, MF_BYCOMMAND);
	HMENU hMenu =  GetSystemMenu( hAppWindow, FALSE );
	int  cMenuItems = GetMenuItemCount(hMenu);
	for(int x = 0; x < (cMenuItems - 1); x++)
	DeleteMenu(hMenu, 0, MF_BYPOSITION);

	if(!CheckLibFuncs()){
		MessageBox(hAppWindow, "CheckLibFuncs() failed.  EXITING APPLICATION", "-- error", MB_OK);
		return 1;}

	mainMenu();
	system("cls");
	return 1;
}


//****************************************************************\
//================================================================*\
//==== SETWIN32CURSORON FUNCTION ==================================*>
//================================================================*/
/*****************************************************************/
void SetWin32CursorOn(void)
{
	ConsoleCursorInfo.bVisible = TRUE;
	SetConsoleCursorInfo(hOutCon, &ConsoleCursorInfo);
}



//****************************************************************\
//================================================================*\
//==== SETWIN32CURSOROFF FUNCTION =================================*>
//================================================================*/
/*****************************************************************/
void SetWin32CursorOff(void)
{
	ConsoleCursorInfo.bVisible = FALSE;
	SetConsoleCursorInfo(hOutCon, &ConsoleCursorInfo);
}


//****************************************************************\
//================================================================*\
//==== CONSOLECTRLHANDLER FUNCTION ================================*>
//================================================================*/
/*****************************************************************/
BOOL ConsoleCtrlHandler(DWORD dwCtrlType)
{
	switch(dwCtrlType)
	{
		case CTRL_BREAK_EVENT: break;
		case CTRL_CLOSE_EVENT: break;
					MessageBox(hAppWindow, "got close event", "-- error", MB_OK);
					break;
		case CTRL_LOGOFF_EVENT: break;
		case CTRL_SHUTDOWN_EVENT: break;
//			CleanupBeforeExit();
		case CTRL_C_EVENT: break;
			break;
	}
	return true;
}

//****************************************************************\
//================================================================*\
//==== SETUPCONSOLE FUNCTION ======================================*>
//================================================================*/
/*****************************************************************/
BOOLEAN SetupConsole()
{
	char cBuff[128];
	hOutCon = GetStdHandle(STD_OUTPUT_HANDLE);

	if(!hOutCon){
		MessageBox(hAppWindow, "Error writing to console", "-- error", MB_OK);
		return 1;}

	hInCon =  GetStdHandle(STD_INPUT_HANDLE);
	if(!hInCon){
		MessageBox(hAppWindow, "Error reading from console", "-- error", MB_OK);
		return 1;}
	
	if(!SetConsoleCtrlHandler((PHANDLER_ROUTINE)ConsoleCtrlHandler, CTRL_BREAK_EVENT))
		MessageBox(hAppWindow, "Error setting up console close handler", "-- error", MB_OK);
	if(!SetConsoleCtrlHandler((PHANDLER_ROUTINE)ConsoleCtrlHandler, CTRL_CLOSE_EVENT))
		MessageBox(hAppWindow, "Error setting up console close handler", "-- error", MB_OK);
	if(!SetConsoleCtrlHandler((PHANDLER_ROUTINE)ConsoleCtrlHandler, CTRL_LOGOFF_EVENT))
		MessageBox(hAppWindow, "Error setting up console close handler", "-- error", MB_OK);
	if(!SetConsoleCtrlHandler((PHANDLER_ROUTINE)ConsoleCtrlHandler, CTRL_SHUTDOWN_EVENT))
		MessageBox(hAppWindow, "Error setting up console close handler", "-- error", MB_OK);

	ConsoleCursorInfo.dwSize = 1; ConsoleCursorInfo.bVisible = TRUE;
	sprintf(cBuff,"  IMPORTANT ->  Use  only    \"Q\"    to  exit  program.    ");
	SetConsoleTitle(cBuff); 

	return TRUE;
}




//****************************************************************\
//================================================================*\
//==== LOADLIBRARIES FUNCTION =====================================*>
//================================================================*/
/*****************************************************************/
BOOLEAN LoadLibraries()
{
		hLibLL_USB = LoadLibrary ("LL_USB2K.DLL");

    if(!hLibLL_USB) return FALSE;
    return TRUE;
}



//****************************************************************\
//================================================================*\
//==== CHECKLIBFUNCS FUNCTION =====================================*>
//================================================================*/
/*****************************************************************/
BOOLEAN
CheckLibFuncs()
{ 
//START OF new 12-2017
//---------->
	SP_GetRegDevList = (P_GET_REG_DEV_LIST)GetProcAddress(hLibLL_USB, "SP_GetRegDevList");
//---------->
	if(!SP_GetRegDevList)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     SP_GetRegDevList(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	SP_GetDevStat = (P_GET_DEV_STAT)GetProcAddress(hLibLL_USB, "SP_GetDevStat");
//---------->
	if(!SP_GetDevStat)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     SP_GetDevStat(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	SP_CheckPower = (P_CHECK_POWER)GetProcAddress(hLibLL_USB, "SP_CheckPower");
//---------->
	if(!SP_CheckPower)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     SP_CheckPower(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	EX_Connect = (P_EX_CONNECT)GetProcAddress(hLibLL_USB, "EX_Connect");
//---------->
	if(!EX_Connect)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     EX_Connect(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	EX_ConnectOneDevice = (P_EX_CONNECT_ONE_DEVICE)GetProcAddress(hLibLL_USB, "EX_ConnectOneDevice");
//---------->
	if(!EX_ConnectOneDevice)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     EX_ConnectOneDevice(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	EX_SendRate = (P_EX_SEND_RATE)GetProcAddress(hLibLL_USB, "EX_SendRate");
//---------->
	if(!EX_SendRate)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     EX_SendRate(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	EX_GetCalculatedRate = (P_EX_GET_CALCULATED_RATE)GetProcAddress(hLibLL_USB, "EX_GetCalculatedRate");
//---------->
	if(!EX_GetCalculatedRate)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     EX_GetCalculatedRate(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	EX_SendChan = (P_EX_SEND_CHAN)GetProcAddress(hLibLL_USB, "EX_SendChan");
//---------->
	if(!EX_SendChan)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     EX_SendChan(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	EX_GetOneConversion = (P_EX_GET_ONE_CONVERSION)GetProcAddress(hLibLL_USB, "EX_GetOneConversion");
//---------->
	if(!EX_GetOneConversion)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     EX_GetOneConversion(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	EX_GetDigin = (P_EX_GET_DIGIN)GetProcAddress(hLibLL_USB, "EX_GetDigin");
//---------->
	if(!EX_GetDigin)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     EX_GetDigin(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	EX_SendDigout = (P_EX_SEND_DIGOUT)GetProcAddress(hLibLL_USB, "EX_SendDigout");
//---------->
	if(!EX_SendDigout)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     EX_SendDigout(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	EX_SendDAC = (P_EX_SEND_DAC)GetProcAddress(hLibLL_USB, "EX_SendDAC");
//---------->
	if(!EX_SendDAC)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     EX_SendDAC(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	SP_SystemCalibration = (P_SP_SYSTEM_CALIBRATION)GetProcAddress(hLibLL_USB, "SP_SystemCalibration");
//---------->
	if(!SP_SystemCalibration)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     SP_SystemCalibration(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	SP_ResetDevice = (P_SP_RESET_DEVICE)GetProcAddress(hLibLL_USB, "SP_ResetDevice");
//---------->
	if(!SP_ResetDevice)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     SP_ResetDevice(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	EX_SetScanType = (P_EX_SET_SCAN_TYPE)GetProcAddress(hLibLL_USB, "EX_SetScanType");
//---------->
	if(!EX_SetScanType)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     EX_SetScanType(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	EX_SetDataLogOptions = (P_EX_SET_DATA_LOG_OPTIONS)GetProcAddress(hLibLL_USB, "EX_SetDataLogOptions");
//---------->
	if(!EX_SetDataLogOptions)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     EX_SetDataLogOptions(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	EX_SetMultiChanScanChans = (P_EX_SET_MULTICHAN_SCAN_CHANS)GetProcAddress(hLibLL_USB, "EX_SetMultiChanScanChans");
//---------->
	if(!EX_SetMultiChanScanChans)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     EX_SetMultiChanScanChans(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	EX_Run = (P_EX_RUN)GetProcAddress(hLibLL_USB, "EX_Run");
//---------->
	if(!EX_Run)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     EX_Run(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	EX_CheckScanStatus = (P_EX_CHECK_SCAN_STATUS)GetProcAddress(hLibLL_USB, "EX_CheckScanStatus");
//---------->
	if(!EX_CheckScanStatus)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     EX_CheckScanStatus(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	EX_GetScanDataDblDigin = (P_EX_GET_SCAN_DATA_DBL_DIGIN)GetProcAddress(hLibLL_USB, "EX_GetScanDataDblDigin");
//---------->
	if(!EX_GetScanDataDblDigin)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     EX_GetScanDataDblDigin(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	EX_GetScanDataDbl = (P_EX_GET_SCAN_DATA_DBL)GetProcAddress(hLibLL_USB, "EX_GetScanDataDbl");
//---------->
	if(!EX_GetScanDataDbl)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     EX_GetScanDataDbl(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	EX_Stop = (P_EX_STOP)GetProcAddress(hLibLL_USB, "EX_Stop");
//---------->
	if(!EX_Stop)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     EX_Stop(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//---------->
	EX_StopComplete = (P_EX_STOP_COMPLETE)GetProcAddress(hLibLL_USB, "EX_StopComplete");
//---------->
	if(!EX_StopComplete)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     EX_StopComplete(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//3-26-18
//---------->
	DLL_ReleaseDeviceFromIDList = (P_DLL_RELEASE_DEVICE_FROM_ID_LIST)GetProcAddress(hLibLL_USB, "DLL_ReleaseDeviceFromIDList");
//---------->
	if(!DLL_ReleaseDeviceFromIDList)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     DLL_ReleaseDeviceFromIDList(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->

//3-26-18
//---------->
	DLL_ReInitOneDev = (P_DLL_REINIT_ONE_DEV)GetProcAddress(hLibLL_USB, "DLL_ReInitOneDev");
//---------->
	if(!DLL_ReInitOneDev)
	{
		MessageBox(GetForegroundWindow(), "failed to find function:\n     DLL_ReInitOneDev(...) \nin LL_USB2K.DLL\n\nEXITING", "IMPORTANT NOTICE ---", MB_OK);
		return FALSE;
	}
//---------->



	return TRUE;
}

//#########################################################################################################################################################
//#########################################################################################################################################################
//#########################################################################################################################################################
//#########################################################################################################################################################
//#########################################################################################################################################################
//#########################################################################################################################################################
BOOLEAN stdscrAddstr(char* str)
{ sprintf(ConBuff, str, usActiveDevID); WRITE_CON return TRUE; }
BOOLEAN stdscrMove(BYTE y, BYTE x) //python reverses these from normal x/y format
{ gotoxy(x,y); return TRUE; }
BOOLEAN stdscrRefresh(void)
{ return TRUE; }
BOOLEAN stdscrClear(void)
{ return TRUE; }
BOOLEAN stdscrErase(UINT n)
{ for(UINT x = 0; x < n; x++) stdscrAddstr(" "); return TRUE; }

BOOLEAN toBinaryString(UINT num)
{
  int i;
  binBuff[8] = '\0';
  for (i = 7; i >= 0; --i, num >>= 1)
  { binBuff[i] = (num & 1) + '0'; }
	return TRUE;
}
   

BOOLEAN getDevID(void)
{
  BOOLEAN fGotValidDev = FALSE;

  //# force reinit if device ID changed 
  USHORT usDevID_temp = 0;
	char strDevIDVal[32];
  
  while(true)
	{
    stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH);
    stdscrRefresh(); stdscrMove(scHeight - 14, 0);
    stdscrAddstr( "Please enter a \"valid\" Device ID  or just <ENTER> to return to menu: ");
		SetWin32CursorOn();   		
		gets_s(strDevIDVal);
		if(strlen(strDevIDVal) == 0)
		{
			//if here, user already decided to cancel and return
			stdscrMove(scHeight - 14, 0); 
			stdscrErase(SCRN_WIDTH); 
			SetWin32CursorOff();   		
			return TRUE;
		}
		break;
	}
	usDevID_temp = atoi(strDevIDVal);
	USHORT usDevList [32];
	UINT uiUnused = 0;

	if(usDevID_temp != usActiveDevID)
	{
		if(!initDone)
		{
			stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
			stdscrRefresh(); stdscrMove(scHeight - 14, 0); 
			stdscrAddstr(" New device ID entered. Reinitialization needed <ENTER> to continue: ");
			getch();
			SetWin32CursorOff();   		
			stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH);
			usActiveDevID = usDevID_temp;
			return TRUE;
		}

		if(SP_GetRegDevList(usDevList, uiUnused))
		{
			for(BYTE x = 0; x < 32; x++)
			{
				if(usDevID_temp == usDevList[x])
				{
					bGotValidDev = 1; 
					usActiveDevID = usDevID_temp; 
					return TRUE; 
				}
			}
			if(!bGotValidDev)
			{
				stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
				stdscrRefresh(); stdscrMove(scHeight - 14, 0); 
				stdscrAddstr("your requested device ID %u was not found - any key to continue: ");
				getch();
				SetWin32CursorOff();   		
				stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
				return FALSE;
			}
			initDone = 0;
			stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
			stdscrRefresh(); stdscrMove(scHeight - 14, 0); 
			stdscrAddstr("you've entered a new ID %u. You'll need to reinitialize - any key to continue: ");
			getch();
			SetWin32CursorOff();   		
			stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH);
			return TRUE;
		}
		else
		{
      stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); stdscrMove(scHeight - 14, 0);
      stdscrAddstr( "Call to DLL to get device ID list failed - any key to continue: ");
      stdscrRefresh();
			SetWin32CursorOn(); 
			getch();
			SetWin32CursorOff();
			return FALSE;
		}
	}
	return TRUE;
}



BOOLEAN doAnalogOut(void)
{
  BYTE bDisplayLineNum;
	char strDacVal[32];
  if(fIsScanning == 0) bDisplayLineNum = (scHeight - 14);
  else bDisplayLineNum = (scHeight - 4); 

	//get DAC
	while(true)
	{
    stdscrMove(bDisplayLineNum, 0); 
		stdscrErase(SCRN_WIDTH); 
		stdscrRefresh(); 
		stdscrMove(bDisplayLineNum, 0);
    stdscrAddstr( "enter DAC (0,1,2) or just <ENTER> to cancel: ");
		SetWin32CursorOn();   		
		gets_s(strDacVal);
		if(strlen(strDacVal) == 0)
		{
			//if here, user already decided to cancel and return
			stdscrMove(bDisplayLineNum, 0); 
			stdscrErase(SCRN_WIDTH); 
			SetWin32CursorOff();   		
			return TRUE;
		}
		if((atoi(strDacVal) > 2) || (atoi(strDacVal) < 0))
		{
			stdscrMove(bDisplayLineNum, 0); 
			stdscrErase(SCRN_WIDTH); 
			stdscrRefresh(); 
			stdscrMove(bDisplayLineNum, 0);
			printf("you entered invalid DAC: %s, any key to try again: ", strDacVal);
			getch();
			continue;
		}
		break;
	}
	bDAC = atoi(strDacVal);
	stdscrMove(bDisplayLineNum, 0); 
	stdscrErase(SCRN_WIDTH); 

	char strAnalogVal[32];
	while(true)
	{
    stdscrMove(bDisplayLineNum, 0); 
		stdscrErase(SCRN_WIDTH); 
		stdscrRefresh(); 
		stdscrMove(bDisplayLineNum, 0);
    stdscrAddstr( "enter voltage (0-5 volts) or just <ENTER> to cancel: ");
		SetWin32CursorOn();   		
		gets_s(strAnalogVal);
		if(strlen(strAnalogVal) == 0)
		{
			//if here, user already decided to cancel and return
			stdscrMove(bDisplayLineNum, 0); 
			stdscrErase(SCRN_WIDTH); 
			SetWin32CursorOff();   		
			return TRUE;
		}
		if((atof(strAnalogVal) > 5) || (atof(strAnalogVal) < 0))
		{
			stdscrMove(bDisplayLineNum, 0); 
			stdscrErase(SCRN_WIDTH); 
			printf("you entered invalid voltage: %s, any key to try again: ", strAnalogVal);
			getch();
			stdscrMove(bDisplayLineNum, 0); 
			stdscrErase(SCRN_WIDTH); 
			continue;
		}
		break;
	}
	dblLastAnalogOut = atof(strAnalogVal);
	EX_SendDAC(usActiveDevID, dblLastAnalogOut, bDAC);

	fAnalogSent = TRUE;
	stdscrMove(bDisplayLineNum, 0); 
	stdscrErase(SCRN_WIDTH); 
	SetWin32CursorOff();   		
	return TRUE;
}


BOOLEAN doDigout(void)
{
  BYTE bDisplayLineNum;
	char strDigoutVal[32];
  if(fIsScanning == 0) bDisplayLineNum = (scHeight - 14);
  else bDisplayLineNum = (scHeight - 4); 
	while(true)
	{
    stdscrMove(bDisplayLineNum, 0); 
		stdscrErase(SCRN_WIDTH); 
		stdscrRefresh(); 
		stdscrMove(bDisplayLineNum, 0);
    stdscrAddstr( "enter digout (0-255) or just <ENTER> to cancel: ");
		SetWin32CursorOn();   		
		gets_s(strDigoutVal);
		if(strlen(strDigoutVal) == 0)
		{
			//if here, user already decided to cancel and return
			stdscrMove(bDisplayLineNum, 0); 
			stdscrErase(SCRN_WIDTH); 
			SetWin32CursorOff();   		
			return TRUE;
		}
		if((atoi(strDigoutVal) > 255) || (atoi(strDigoutVal) < 0))
		{
			printf("\r\nyou entered invalid digital output: %s\r\n", strDigoutVal);
			printf("\r\nany key to try again: ");
			getch();
			continue;
		}
		break;
	}
	bDigout = atoi(strDigoutVal);
	EX_SendDigout(usActiveDevID, bDigout);
	fDigoutSent = TRUE;
	stdscrMove(bDisplayLineNum, 0); 
	stdscrErase(SCRN_WIDTH); 
	SetWin32CursorOff();   		
	return TRUE;
}

BOOLEAN doDigin(void)
{
  BYTE bDiginTemp;
  if(EX_GetDigin(usActiveDevID, &bDiginTemp))
	{
    fDiginRead = TRUE;
    bDigin = bDiginTemp;
    if((usActiveDevID > 100) && (usActiveDevID < 5000))bDigin ^= 255; //  # invert for Model 301 hardware compatibility
		return TRUE;
	}
  return FALSE;
}

BOOLEAN doGetConv(void)
{
  DOUBLE dblMilliVolts = 0;
  if(EX_GetOneConversion(usActiveDevID, &dblMilliVolts))
	{
		fGotAnyConversion = TRUE;
		dblLastConversion = dblMilliVolts / 1000;
		return TRUE;
	}
	return FALSE;
}



BOOLEAN  doSendChan(void)
{
  BYTE bDisplayLineNum = 0;
  if(fIsScanning == 0) bDisplayLineNum = 39;
  else bDisplayLineNum = (scHeight - 4); 
  BYTE pbDigin =  0;
  BYTE bChan_TEMP = 0;
	char strChanVal[32];
  while(true)
	{
		system("cls");
		SetWin32CursorOn();   		
		printf("\r\n\r\n\r\n");
		printf("enter chan (0-7) then press <ENTER> or \"just\" <ENTER> to cancel and exit: ");
		gets_s(strChanVal);
		if(strlen(strChanVal) == 0)
		{
			//if here, user already decided to cancel and return
			system("cls");
			return TRUE;
		}
		if((atoi(strChanVal) > 8) || (atof(strChanVal) < 0))
		{
			printf("\r\nyou entered invalid channel: %s\r\n", strChanVal);
			printf("\r\nany key to try again: ");
			getch();
			continue;
		}
		break;
	}
  if(initDone == 1)
	{
		BYTE bDigInTemp = 0;
		bChan_TEMP = atoi(strChanVal);

		printf("\r\nsending channel: %u, wait . . .\r\n", bChan_TEMP);
    if(EX_SendChan(usActiveDevID, bChan_TEMP, &bDigInTemp))
		{
      bCurChan = bChan_TEMP; 
			//bDigin = bDigInTemp;
			fChanSent =  TRUE;
			printf("\r\nsend-chan success\r\n");
			printf(" any key to continue: ");
			SetWin32CursorOn();   		
			getch();
			system("cls");
			SetWin32CursorOff();   		
			return TRUE;
		}
		else
		{
			printf("\r\nsend-chan failed\r\n");
			printf(" any key to continue: ");
			SetWin32CursorOn();   		
			getch();
			system("cls");
			SetWin32CursorOff();   		
			return TRUE;
		}
	}

	printf("\r\nNot yet signed on.\r\n will send channel: %u during signon\r\n", atoi(strChanVal));
	printf("\r\nany key to try again: ");
	getch();
	SetWin32CursorOff();   		
	system("cls");
	return TRUE;
}

BOOLEAN doGetActualRate(void)
{
	double dblRate_TEMP = 0;
  dblRate_TEMP = dblRate;
  BYTE bDisplayLineNum = 39; 
	if(!EX_GetCalculatedRate(usActiveDevID, &dblRate_TEMP))
	{
    stdscrMove(bDisplayLineNum, 0); 
		stdscrErase(SCRN_WIDTH); 
    stdscrMove(bDisplayLineNum, 0); 
		SetWin32CursorOn();   		
		printf("failed gettting \"actual\" rate from DLL - any key to continue: ");
		getch();
		SetWin32CursorOff();   		
		return FALSE;
	}
	dblActualRate = dblRate_TEMP;
	return TRUE;
}

BOOLEAN doSendRate(void)
{
	char strFloatVal[32];
	double dblRate_TEMP = 0;

  if(bSendRateQuiet == 0)
	{
		while(TRUE)
		{
			stdscrMove(scHeight - 15, 0); stdscrErase(SCRN_WIDTH); 
			stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
			stdscrMove(scHeight - 15, 0); 
			stdscrAddstr( "enter rate value between 50 and 1000\r\n");
			printf("  then press <ENTER> or \"just\" <ENTER> to cancel and exit: ");
			SetWin32CursorOn();   		
			gets_s(strFloatVal);
			if(strlen(strFloatVal) == 0)
			{
				stdscrMove(scHeight - 15, 0); stdscrErase(SCRN_WIDTH); 
				stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
				SetWin32CursorOff();   		
				return TRUE;
			}
			if((atof(strFloatVal) > 1000) || (atof(strFloatVal) < 50))
			{
				stdscrMove(scHeight - 15, 0); stdscrErase(SCRN_WIDTH); 
				stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
				stdscrMove(scHeight - 15, 0); 
				printf("\r\nyou entered invalid rate: %s\r\n", strFloatVal);
				printf("\r\nany key to try again: ");
				getch();
				continue;
			}
			break;
		}
		dblRate_TEMP = atof(strFloatVal);
		SetWin32CursorOff(); 
    if(initDone)
		{
			stdscrMove(scHeight - 15, 0); stdscrErase(SCRN_WIDTH); 
			stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
			stdscrMove(scHeight - 14, 0); 
			printf("\r\nsending rate: %4.4f\r\n", atof(strFloatVal));
      if(EX_SendRate(usActiveDevID, &dblRate_TEMP))
			{
        dblRate = dblRate_TEMP; //re-use dblRate_TEMP below in different context below
				printf(" calculating \"actual rate\" which may vary slightly from \"requested\" rate\r\n");
				if(!EX_GetCalculatedRate(usActiveDevID, &dblRate_TEMP))
				{
					SetWin32CursorOn();   		
					printf(" failed gettting \"actual\" rate from DLL"); 
					printf(" any key to continue: ");
					getch();
					SetWin32CursorOff();   		
					system("cls");
					return FALSE;
				}
				dblActualRate = dblRate_TEMP;
				printf(" got \"actual\" rate: %4.4f - any key to continue: ", dblActualRate);
				SetWin32CursorOn();   		
				getch();
				system("cls");
				SetWin32CursorOff();   		
				return TRUE;
			}
			SetWin32CursorOn();   		
			printf(" failed sending rate - <ENTER> to continue: "); 
			getch();
			SetWin32CursorOff();   		
			system("cls");
      return FALSE;
		}
		else
		{
			stdscrMove(scHeight - 15, 0); stdscrErase(SCRN_WIDTH); 
			stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
			stdscrMove(scHeight - 15, 0); 
			SetWin32CursorOn();   		
      dblRate = dblRate_TEMP;
			dblActualRate = dblRate;
			printf("rate: %4.4f will be set when board is initialized\r\n", dblRate);
			printf(" any key to continue: ");
			getch();
			SetWin32CursorOff();   		
			system("cls");
			return TRUE;
		}
	}
	//NOTE: all lines above exit so no "else" needed here
  if(initDone)
	{
    if(EX_SendRate(usActiveDevID, &dblRate_TEMP))
		{
      dblRate = dblRate_TEMP; //re-use dblRate_TEMP below in different context below
			if(!EX_GetCalculatedRate(usActiveDevID, &dblRate_TEMP))
				return FALSE;  //silent mode
			dblActualRate = dblRate_TEMP;
			fRateSent = TRUE;
			return TRUE;
		}
    return FALSE;
	}
	else
	{
      dblRate = dblRate_TEMP; 
			return TRUE;
	}
}

BOOLEAN displayDevList(void)
{
  stdscrMove(scHeight - 14, 0); 
	stdscrErase(SCRN_WIDTH); 
	stdscrRefresh(); 
	stdscrMove(scHeight - 14, 0);
  stdscrAddstr( "Dev IDs found:  ");
	BOOLEAN fFirstFound = FALSE;
  for(int x = 0; x < 32; x++)
	{
    if( ((DevListSave[x] & 0x8000) != 0x8000) && (DevListSave[x] != 0))
		{
      if(!fFirstFound)
			{
				sprintf(cBigBuff,  "%4d ", DevListSave[x]); 
				stdscrAddstr(cBigBuff); 
        fFirstFound = 1;
			}
      else
			{
				sprintf(cBigBuff,  ", %4d ", DevListSave[x]); 
				stdscrAddstr(cBigBuff); 
			}
		}
	}
	if(!fFirstFound) initDone = 0;
	return TRUE;
}


BOOLEAN getDevList(void)
{
  UINT uiUnused = 0;
  if(SP_GetRegDevList(DevList, uiUnused))
    for(int x = 0; x < 32; x++) DevListSave[x] = DevList[x];
	return TRUE;
}

BOOLEAN getDevConnectStat()
{
  //global initDone
	#define DEV_NOT_FOUND         0x0
	#define DEV_FOUND             0x1
	#define DEV_DISCONNECTED      0x2
	#define DEV_ENUMERATED        0x4
	#define DEV_READY             0x8
	#define DEV_OWNED_BY_ANOTHER  0x10
	#define DEV_UNKNOWN_ERR       0x20
    
	UINT uiUnused;
	BOOLEAN fGotValidDev = FALSE; 
	UINT uiDevStat = 0;

  SP_GetDevStat(usActiveDevID, &uiDevStat);
  if((uiDevStat & 0x1) != 1)
	{
		if(initDone)
		{
			stdscrMove(scHeight - 14, 0); 
			stdscrErase(SCRN_WIDTH); 
			stdscrRefresh(); stdscrMove(scHeight - 14, 0);
			sprintf(cBigBuff,  "device %u no longer connected. Any key to continue: ", usActiveDevID); stdscrAddstr(cBigBuff); 
			getch();
			stdscrRefresh();
		}
		initDone = FALSE;
    return FALSE;
	}
  else if((uiDevStat & 0x4) != 0x4)
	{
			SP_GetRegDevList(DevList, uiUnused);
			for(int x = 0; x < 32; x++)
			{
				if(usActiveDevID == DevList[x]){ bGotValidDev = 1; break; }
			}
      if(bGotValidDev)
			{
				if(initDone)
				{
					stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); stdscrMove(scHeight - 14, 0);
					stdscrAddstr( "device now connected but needs re-initialized");
					stdscrRefresh();
					Sleep(1000);
				}
			}
      else
			{
				if(initDone)
				{
					stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); stdscrMove(scHeight - 14, 0);
					sprintf(cBigBuff,  "your device %u no longer connected ", usActiveDevID); stdscrAddstr(cBigBuff); 
				}
        initDone = 0;
        stdscrRefresh();
        Sleep(2000);
        stdscrRefresh();
			}
      return FALSE;
	}
  else
	{
    UINT uiFlags = 0;
    BYTE bLastDigin = 0;
		SP_CheckPower(usActiveDevID, bCurChan, &bLastDigin, &uiFlags);

    if(uiFlags != 1)
		{
			if(initDone)
			{
				stdscrMove(40, 0); stdscrErase(SCRN_WIDTH); stdscrRefresh();
				stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); stdscrMove(scHeight - 14, 0); 
				stdscrAddstr(" You may have lost USB connection or power to this device\n Reinitialization needed <ENTER> to continue: ");
				stdscrRefresh(); 
				SetWin32CursorOn();   		
				getch();
				for(int x = 17; x < 52; x++){stdscrMove(x, 0); stdscrErase(SCRN_WIDTH);}
				SetWin32CursorOff();   		
				stdscrRefresh();
			}
      initDone = 0;
		}
	}
	return TRUE;
}

BOOLEAN doInit(void)
{
	system("cls");

  stdscrMove(2, 0); 
	stdscrAddstr(" attempting to find and connect to a device");
  stdscrMove(3, 0); 
	stdscrAddstr("  please wait . . .");

	// EX_ConnectOneDevice(...) or EX_Connect(...) can be used
	if (!EX_ConnectOneDevice(usActiveDevID, dblRate, bCurChan))
	{
		SP_ResetDevice(usActiveDevID);
		if (!EX_ConnectOneDevice(usActiveDevID, dblRate, bCurChan))
		//if(!EX_Connect(&usActiveDevID))
		{
			system("cls");
			usActiveDevID = 0;
			stdscrMove(2, 0); 
			stdscrAddstr("Failed to connect!");
			stdscrMove(3, 0); 
			stdscrAddstr(" not signed on");
			stdscrMove(4, 0); 
			stdscrAddstr(" any key to continue: ");
			getch();
			SetWin32CursorOff();   		
			system("cls");
			return FALSE;
		}
	}
  stdscrMove(5, 0); 
	char buff[32];
	sprintf(buff, " connected to %u", usActiveDevID);
	stdscrAddstr(buff);
	DOUBLE dblRate_TEMP = dblRate;
	if(!EX_GetCalculatedRate(usActiveDevID, &dblRate_TEMP))
	{
		stdscrMove(8, 0); 
		stdscrAddstr("  Failed call - any key to continue: ");
		SetWin32CursorOn();   		
		getch();
		SetWin32CursorOff();   		
		system("cls");
		return FALSE; 
	}
	dblActualRate = dblRate_TEMP;
	fRateSent = TRUE;
	stdscrMove(7, 0); 
	sprintf(buff, " got \"actual rate\": %3.5lf", dblActualRate);
	stdscrAddstr(buff);
	if(fGotDevError_global)
	{
		initDone = TRUE;
		system("cls");
		return TRUE;
	}

  stdscrMove(9, 0); 
	SetWin32CursorOn();   		
	stdscrAddstr(" success - any key to continue: ");
	getch();
	initDone = TRUE;
	system("cls");
	return TRUE;
}


BOOLEAN writeConfig(BYTE bQuiet)
{
	FILE *fpConfig;
	fpConfig = fopen("LL_USB.CFG", "w");
	if(fpConfig == NULL)
	{
		stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); 
		stdscrMove(scHeight - 14, 0);
		stdscrAddstr( "could not open/write to config file - any key to continue: ");
		SetWin32CursorOn();   		
		getch();
		SetWin32CursorOff(); 
		return FALSE;
	}
	fprintf(fpConfig, "To use this file use the format shown below. First enter the device ID number (replace\n" 
		"the one shown below with your own), followed by the rate you'd like to run, followed by\n" 
		"the channel. All that is really required is the device ID which if entered with nothing\n" 
		"else, will have default values of 100Hz and Channel 7 set by the application. Note that\n" 
		"the  ' = '  sign follows each of the values with no space separation. Invalid entries.\n" 
		"will be either ignored or replaced by default values.\n\n" );
	fprintf(fpConfig, "ID=%u\n", usActiveDevID);
	fprintf(fpConfig, "RATE=%3.2lf\n", dblRate);
	fprintf(fpConfig, "CHAN=%u\n", bCurChan);
	fclose(fpConfig);
	return(FALSE);
}


BOOLEAN readConfig(BYTE bQuiet)
{
	char buffString[3][32];
	FILE *fpConfig;
	int line_num = 1;
	int find_result = 0;
	char temp[512];
	char fName[32];
	sprintf(fName, "LL_USB.cfg");
	sprintf(buffString[0], "ID=");
	sprintf(buffString[1], "RATE=");
	sprintf(buffString[2], "CHAN=");

	//set defaults
	usActiveDevID = 0;
	dblRate = 100;
	bCurChan = 0;

	if(!bQuiet) stdscrMove(26, 0); 
	for(int x = 0; x < 3; x++)
	{
		line_num = 0; find_result = 0;
		fpConfig = fopen(fName, "r");
		if(fpConfig == NULL) return(FALSE);

		while(fgets(temp, 512, fpConfig) != NULL) 
		{
			if((strstr(temp, buffString[x])) != NULL) 
			{
				if(x == 0)
				{ usActiveDevID = atoi(&temp[strlen(buffString[x])]); if(!bQuiet)printf("\ndev:  %u", usActiveDevID); }
				if(x == 1)
				{ dblRate = atof(&temp[strlen(buffString[x])]); if(!bQuiet)printf("\nrate: %3.2lf", dblRate); dblActualRate = dblRate;}
				if(x == 2)
				{ bCurChan = atoi(&temp[strlen(buffString[x])]); if(!bQuiet)printf("\nchan: %u", bCurChan); }
				find_result++;
			}
			line_num++;
		}
		if(find_result == 0)  if(!bQuiet)printf("\nSorry, couldn't find a match.\n");
		if(fpConfig) fclose(fpConfig);
	}
	return TRUE;
}


BOOLEAN mainMenu(void)
{
  initDone = FALSE;
  usActiveDevID = 0;
	UINT uiUnused;
	if(!readConfig(0))
	{
		stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); 
		stdscrMove(scHeight - 14, 0);
		stdscrAddstr( "could not open/read config file - any key to continue: ");
		SetWin32CursorOn();   		
		getch();
		SetWin32CursorOff(); 
	}
  
  if(usActiveDevID != 0)
	{
    BOOLEAN fFound = FALSE;
    if(SP_GetRegDevList(DevList, uiUnused))
      for(int x = 0; x < 32; x++) if(usActiveDevID == DevList[x]){ fFound = 1; break; }

    if(!fFound)
		{
      getDevList(); 
			displayDevList();
		}
	}
	initDone = FALSE;

  while(fMasterDone == 0)
	{
    stdscrClear();
    stdscrRefresh();
    stdscrMove(4, 22); stdscrAddstr("------------ MAIN MENU --------------");
		if(initDone == 0)
		{ stdscrMove(13, 0); stdscrErase(SCRN_WIDTH); stdscrMove(13, 22); }
    stdscrRefresh();
    getDevList(); displayDevList();
    stdscrMove(7, 22); 
		stdscrAddstr( "Q) Quit Program");
    stdscrMove(9, 22);
		if(usActiveDevID == 0) stdscrAddstr( "D) Device ID (must be done first)   ");
		else { sprintf(cBigBuff, "D) Device ID: %d          ", usActiveDevID); stdscrAddstr(cBigBuff); }

    stdscrMove(11, 22);
		if(fRateSent) sprintf(cBigBuff,  "R) Set internal data rate: %3.3lfHz sent, actual rate: %3.3lfHz      ", dblRate, dblActualRate); 
		else sprintf(cBigBuff,  "R) Set internal data rate: %3.3lfHz will send when initialized ", dblRate); 
		stdscrAddstr(cBigBuff); 

    stdscrMove(13, 22);
		if(fChanSent) sprintf(cBigBuff,  "C) Channel %d sent                          ", bCurChan);
		else sprintf(cBigBuff,  "C) Channel %d  will send when initialized", bCurChan);   
		stdscrAddstr(cBigBuff);

    stdscrMove(15, 0); stdscrErase(SCRN_WIDTH);  
    stdscrMove(15, 22); stdscrAddstr( "I) Initialize Device");
    stdscrRefresh();

    if(initDone)
		{
      stdscrMove(15, 22); stdscrAddstr( "I) Initialize Device (initialized)"); 
			if(!fGotAnyConversion){ stdscrMove(17, 22); stdscrAddstr( "G) Get one conversion"); }
			else
			{
				stdscrMove(17, 22); 
				sprintf(cBigBuff,  "G) Get one conversion. Last: %2.5lf", dblLastConversion); 
				stdscrAddstr(cBigBuff); 
			}
      stdscrMove(19, 22);
      if(fDiginRead)
			{
				toBinaryString(bDigin);
				sprintf(cBigBuff,  "N) Digital input (last read: %d)  %s          ", bDigin, binBuff);
				stdscrAddstr(cBigBuff);
			}
      else stdscrAddstr( "N) Digital input                           ");

      stdscrMove(21, 22);
      if(fDigoutSent)
			{
				toBinaryString(bDigout);
				sprintf(cBigBuff,  "O) Digital output (last sent: %d)   %s    " , bDigout, binBuff);
				stdscrAddstr(cBigBuff);
			}
      else stdscrAddstr( "O) Digital output                        ");
      stdscrMove(23, 22);
      if(fAnalogSent)
			{
				sprintf(cBigBuff,  "A) Analog output (last: %1.3f, DAC: %d) ", dblLastAnalogOut, bDAC);
				stdscrAddstr(cBigBuff);
			}
      else stdscrAddstr( "A) Analog output");
      stdscrMove(25, 22); stdscrAddstr( "Y) System Calibration");
      stdscrMove(27, 22); stdscrAddstr( "J) Load conguration");
      stdscrMove(29, 22); stdscrAddstr( "K) Save Configuration");
      stdscrMove(31, 22); stdscrAddstr( "S) Scan Menu");
      stdscrRefresh();
		}

		int keyhit = ' ';
		clock_t uiEndTime = clock() + 1000; 
		while (true)
		{  
			keyhit = ' ';
			if(!kbhit())
			{
				if(initDone)
				{
					if(clock() > uiEndTime)
					{ 
						getDevConnectStat(); 
						if(!initDone) { system("cls"); break; }
						getDevList(); displayDevList();
						uiEndTime = clock() + 1000; 
					}
				}
				else
				{
						if(clock() > uiEndTime)
						{ getDevList(); displayDevList(); uiEndTime = clock() + 1000; }
				}
				continue;
			}
			keyhit = toupper((char)getch());
			if (keyhit == 'D') { getDevID(); break; }
			if (keyhit == 'Q')
			{
        fMasterDone = 1;		//back to main and then exit the app
				system("cls");
				printf("\r\n\r\n\r\n will be disconnecting from device\r\n");
				printf("\r\n press any key and then wait for program to exit: ");
				SetWin32CursorOn();
				getch();
				system("cls");
        return (TRUE);
			}
			if ((keyhit == 'I') && (usActiveDevID != 0))
			{ 
				doInit(); SetWin32CursorOff(); 
				break; 
			} 
			if (keyhit == 'L') 
			{
				getDevList();
				BOOLEAN fListChange = FALSE;
				for(int x = 0; x < 32; x++)
				{ 
					if(DevListSave[x] != DevListNew[x]){ fListChange = TRUE; DevListNew[x] = DevListSave[x]; }
				}
				if(fListChange == 1) break;
			}
			if (keyhit == 'E')
			{ 
				stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
				stdscrMove(scHeight - 14, 0);
				printf("resetting device - wait . . .");
				if(!SP_ResetDevice(usActiveDevID))
				{
					stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); 
					stdscrMove(scHeight - 14, 0);
					stdscrAddstr( "could not reset device - any key to continue: ");
					SetWin32CursorOn();   		
					getch();
					SetWin32CursorOff(); 
				}
				stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
				stdscrMove(scHeight - 14, 0);
				break;
			}
			if (keyhit == 'J')
			{ 
				USHORT usActiveDevID_temp = usActiveDevID;
				DOUBLE dblRate_temp = dblRate;
				BYTE bCurChan_temp = bCurChan;
				if(!readConfig(1)) // 1 prevents displaying polled screen as it does during startup
				{
					stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); 
					stdscrMove(scHeight - 14, 0);
					stdscrAddstr( "could not open/read config file - any key to continue: ");
					SetWin32CursorOn();   		
					getch();
					SetWin32CursorOff(); 
					break;
				}
				doGetActualRate(); 
				getDevList(); 
				displayDevList(); 
				if(usActiveDevID_temp != usActiveDevID)
				{
					stdscrMove(scHeight - 15, 0); stdscrErase(SCRN_WIDTH); 
					stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
					stdscrMove(scHeight - 15, 0);
					SetWin32CursorOn();   		
					printf("DevID from config is different from one you already signed on to.");
					stdscrMove(scHeight - 14, 0);
					printf(" will attempt to initialize this device - any key to continue: ");
					getch();
					if(!doInit())
					{
						initDone = 0;
						stdscrMove(scHeight - 15, 0); stdscrErase(SCRN_WIDTH); 
						stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
						stdscrMove(scHeight - 14, 0);
						printf("New DevID failed to initialize - any key to continue: ");
						getch();
						break;
					}
				}
				if(dblRate_temp != dblRate)
				{
					stdscrMove(scHeight - 15, 0); stdscrErase(SCRN_WIDTH); 
					stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
					stdscrMove(scHeight - 15, 0);
					SetWin32CursorOn();   		
					printf("Rate from config is different from one you already set");
					stdscrMove(scHeight - 14, 0);
					printf(" will attempt to resend rate - any key to continue: ");
					getch();
					bSendRateQuiet = 1;
					if(!doSendRate())
					{
						stdscrMove(scHeight - 15, 0); stdscrErase(SCRN_WIDTH); 
						stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
						stdscrMove(scHeight - 14, 0);
						SetWin32CursorOn();   		
						printf("failed sending rate - any key to continue: ");
						getch();
						stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
						SetWin32CursorOff(); 
						break;
					}
					stdscrMove(scHeight - 15, 0); stdscrErase(SCRN_WIDTH); 
					stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
				}
				if(bCurChan_temp != bCurChan)
				{
					stdscrMove(scHeight - 15, 0); stdscrErase(SCRN_WIDTH); 
					stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
					stdscrMove(scHeight - 15, 0);
					SetWin32CursorOn();   		
					printf("Chan from config is different from one you already set");
					stdscrMove(scHeight - 14, 0);
					printf(" will attempt to resend chan - any key to continue: ");
					getch();
					bSendRateQuiet = 1;
					if(!doSendChan())
					{
						stdscrMove(scHeight - 15, 0); stdscrErase(SCRN_WIDTH); 
						stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
						stdscrMove(scHeight - 14, 0);
						SetWin32CursorOn();   		
						printf("failed sending channel - any key to continue: ");
						getch();
					}
				}
				stdscrMove(scHeight - 15, 0); stdscrErase(SCRN_WIDTH); 
				stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
				stdscrMove(scHeight - 14, 0);
				SetWin32CursorOff();   		
				printf("success");
				break; 
			}
			if(keyhit == 'K'){ writeConfig(0); break; }
			if(keyhit == 'R') { doSendRate(); break; }
      if(keyhit == 'C') { doSendChan(); break; }
			if(!initDone) continue;
      
			if (keyhit == 'S')
			{ 
				scanMenu(); 
				BOOLEAN fDisplayedNoticeOnce = FALSE;
				BOOLEAN fScanLoopBreak = FALSE;
				while(true)
				{
					if(fGotDevError_global)
					{
						if(!fDisplayedNoticeOnce)
						{
							stdscrMove((scHeight - 4), 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); 
							stdscrMove((scHeight - 4), 0); stdscrAddstr("any key to stop trying to restart scan");
							fDisplayedNoticeOnce = TRUE;
						}

						if(kbhit())
						{
							getch();
							fGotDevError_global = false;
							initDone = false;
							return TRUE;
						}

						if (!EX_ConnectOneDevice(usActiveDevID, dblRate, bCurChan))
						{ continue; }
						EX_SendDAC(usActiveDevID, dblLastAnalogOut, bDAC); 
						EX_SendDigout(usActiveDevID, bDigout); 
						system("cls"); 
						scanMenu(); 
					}
					else{ fScanLoopBreak = TRUE; break; }
				}
				if(fScanLoopBreak){ fScanLoopBreak = FALSE; break; }
			}
			if(keyhit == 'G'){ doGetConv(); break; }
			if(keyhit == 'N'){ doDigin(); break; }
			if(keyhit == 'O'){ doDigout(); break; }
			if(keyhit == 'A'){ doAnalogOut(); break; }
      if(keyhit == 'Y')
			{
				stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
				stdscrMove(scHeight - 14, 0);
				printf("calibrating - wait . . .");
				BYTE bCurChan_SAVE = bCurChan;
				if(!SP_SystemCalibration(usActiveDevID, bCurChan))
				{
					SetWin32CursorOn();   		
					printf("failed system calibration - any key to continue: ");
					getch();
				}
				stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH); 
				SetWin32CursorOff(); 
				break;
			}
			if(initDone == 0) 
			{
				system("cls");  //3-15-18
				break;
			}
		}
	}
  return TRUE;
}


BOOLEAN scanMenu(void)
{
  stdscrClear();
  stdscrRefresh();
  while(true)
	{
    for(BYTE x= 0; x < 50; x++)
		{ stdscrMove(x, 0); stdscrErase(SCRN_WIDTH); } 
    stdscrRefresh(); 
    
    if(bScanType >= 4) fDig = 1;											//flag for Digital Input Scan
    else fDig = 0;
    if((bScanType == 3) || (bScanType == 6))fCal = 1;	//flag for Cal Scan  
    else fCal = 0;

    if((bScanType == CMND_SINGLE_CHAN_DIGIN_SCAN) || (bScanType == CMND_SINGLE_CHAN_SCAN))
		{
      fSingle = 1;								//flag for single channel scan
      bNumChansTemp = bNumChans;	//preserve channel count for multichannel scans
      bNumChans = 1;
		}
    else
		{
      fSingle = 0;
      bNumChans = bNumChansTemp;	//save for possible later multi channel scan 
		}

    stdscrMove(2, 22); stdscrAddstr("Model 30x USB Data Acquisition System ");
    stdscrMove(4, 22); stdscrAddstr("--------- S C A N    M E N U ---------");
    stdscrMove(7, 22); stdscrAddstr( "Q) Quit Program");
    stdscrMove(9, 22); stdscrAddstr( "B) Back to Main Menu");
		sprintf(cBigBuff,  "T) Scan Type %s ", stypestr[bScanType]);
    stdscrMove(11, 22); stdscrAddstr(cBigBuff);
    stdscrMove(13, 22); stdscrAddstr("N) Number of channels to scan: ");
    if(fSingle) stdscrAddstr("1  ");
    else
		{
			if(fCal) { sprintf(cBigBuff,  " %d (+ offset)", bNumChans); stdscrAddstr(cBigBuff); }
      else  { sprintf(cBigBuff,  " %d  ", bNumChans); stdscrAddstr(cBigBuff); }
		}
    stdscrMove(15, 22); stdscrAddstr( "L) Log scan data to file ");
    if(fDoLog)stdscrAddstr( "ON ");
    else stdscrAddstr( "OFF");
    stdscrMove(17, 22);
		sprintf(cBigBuff,  "C) Channel number for single-chan scans (currently: %d)  ", bCurChan);
    stdscrAddstr(cBigBuff);
    stdscrMove(19, 22);
		sprintf(cBigBuff,  "R) Set internal data rate (currently %3.3fHz)  ", dblActualRate);
    stdscrAddstr(cBigBuff);
    stdscrMove(20, 22);
    if(fSingle) stdscrAddstr( "    scan rate = internal data rate ");
    else 
		{ 
			sprintf(cBigBuff,  "    scan rate = internal data rate / (5 * numChans) = %3.3fHz ", (dblActualRate/((bNumChans + fCal) * 5)));
			stdscrAddstr(cBigBuff);
		}
    stdscrMove(22, 22); stdscrAddstr( "O) Digital output");
    stdscrMove(24, 22); stdscrAddstr( "A) Analog output");
    stdscrMove(26, 22); stdscrAddstr( "X) Screen display ");
    if(fDoScreen) stdscrAddstr( "ON ");
    else stdscrAddstr( "OFF");

    stdscrMove(28, 22); 
		sprintf(cBigBuff,  "V) Local average: %d   ", bLocAvg); 
		stdscrAddstr(cBigBuff); 

    stdscrMove(30, 22); stdscrAddstr( "S) Start scan");
    stdscrRefresh();

		clock_t uiEndTime = clock() + 1000; 
		int keyhit = ' ';
		while (true)
		{ 
//3-26-18 ###############
			if(fGotDevError_global)
			{
				fGotDevError_global = FALSE;
				keyhit = 'S';
				goto DO_THE_SCAN;
			}




			keyhit = ' ';
			if(!kbhit())
			{
				if(initDone)
				{
					if(clock() > uiEndTime)
					{ 
						getDevConnectStat(); clock_t uiEndTime = clock() + 1000; 
					}
				}
//3-27-18d #########
				else
				{
					system("cls");
					return TRUE;
				}
//##################

				continue;
			}
			keyhit = toupper((char)getch());


//3-27-18d #########
			if(!initDone)
			{
				system("cls");
				return TRUE;
			}
//##################




      if(keyhit == 'Q')
			{
        fMasterDone = 1;		//back to main and then exit the app
				system("cls");
				printf("\r\n\r\n\r\n will be disconnecting from device\r\n");
				printf("\r\n press any key and then wait for program to exit: ");
				SetWin32CursorOn();
				getch();
				system("cls");
        return (FALSE);
			}
      if (keyhit == 'B'){ system("cls"); return (0); }
			if (keyhit == 'T'){ setScanType(); break; }
      if (keyhit == 'N'){ setNumScanChans(); break; }
			if (keyhit == 'L'){ fDoLog ^= 1; break; }
      if (keyhit == 'C'){ doSendChan(); break; }
      if (keyhit == 'R'){ doSendRate(); break; }
      if (keyhit == 'O'){ doDigout(); break; }
      if (keyhit == 'A'){ doAnalogOut(); break; }
      if (keyhit == 'X'){ fDoScreen ^= 1; break; }
      if (keyhit == 'V')
			{ 
				setLocalAvg(); 
				break; 
			} 


DO_THE_SCAN:
      if (keyhit == 'S')
			{
				fGotDevError_global = false;
        scanFunc();

				//3-27-18d
				if(!initDone) 
				{
					system("cls"); 
					return TRUE;
				}
//3-26-18 ##############
				if(fGotDevError_global)
				{
//3-27-18					system("cls"); 
					return (0);
				}

        stdscrMove(scHeight - 2, 0);; stdscrErase(SCRN_WIDTH); stdscrRefresh();
        stdscrMove(scHeight - 2, 1); stdscrAddstr(" Done scan <ENTER> to go back to Scan Menu: ");

				SetWin32CursorOn(); 
				getch();
				SetWin32CursorOff(); 
				system("cls");
        stdscrRefresh();
				fIsScanning = FALSE;
        break;
			}
		}  
	}
	return TRUE;
}


BOOLEAN setScanType(void)
{
  BYTE bScanTypeTemp = 0;
  for(BYTE x = 0; x < 50; x++)
	{
    stdscrMove(x, 0);
    stdscrErase(SCRN_WIDTH); 
	}
  stdscrRefresh(); 
  stdscrMove(12, 0);
  for(BYTE x = 1; x < 7; x++)
	{
		printf(" %d =  ", x);
		printf("%s \n", stypestr[x]);
	}
  stdscrRefresh();  

  char tBuff[2];
  while(true)
	{
		stdscrMove(19, 0); stdscrErase(SCRN_WIDTH); stdscrRefresh();
    stdscrMove(19, 0); stdscrAddstr ("Enter scan type number: ");
    stdscrRefresh(); 
		SetWin32CursorOn();   		
		tBuff[0] = (char)getch();
		if((atoi(tBuff) < 1) || (atoi(tBuff) > 6))
		{
			stdscrMove(19, 0); stdscrErase(SCRN_WIDTH); stdscrRefresh();
			stdscrMove(19, 0); 
			printf(" %u is invalid. Try again? (y/n): ", atoi(tBuff));
			tBuff[0] = toupper((char)getch());
			if(tBuff[0] == 'Y') continue;
			SetWin32CursorOff();
			system("cls");
			return FALSE;
		}
		break;
	}
  bScanType = atoi(tBuff);
	SetWin32CursorOff();
	system("cls");
  return TRUE;
}


BOOLEAN setNumScanChans(void)
{
  if(fSingle)
	{
    stdscrMove(40, 0);
    stdscrAddstr( "always 1 channel when doing a single-channel scan");
    stdscrRefresh();
    return TRUE;
	}

	char tBuff[2];
  while(true)
	{
    stdscrMove(40, 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); stdscrMove(40, 0);
    if(fCal) stdscrAddstr( "enter number of channels to scan (not including offset channel): ");
    else stdscrAddstr( "enter number of channels to scan (1-8): ");
    SetWin32CursorOn();
    stdscrRefresh();
		tBuff[0] = (char)getch();
		if((atoi(tBuff) < 1) || (atoi(tBuff) > 8))
		{
			stdscrMove(19, 0); stdscrErase(SCRN_WIDTH); stdscrRefresh();
			stdscrMove(19, 0); 
			printf(" %u is invalid. Try again? (y/n): ", atoi(tBuff));
			getch();
			tBuff[0] = toupper((char)getch());
			if(tBuff[0] == 'Y') continue;
			SetWin32CursorOff();
			system("cls");
			return FALSE;
		}
		break;
	}
  bNumChans = atoi(tBuff);
	bNumChansTemp = bNumChans;
	SetWin32CursorOff();
	system("cls");
  return TRUE;
}

BOOLEAN setLocalAvg(void)
{
	char strLocalAvg[32];
  while(TRUE)
	{
    stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH);
    stdscrRefresh(); stdscrMove(scHeight - 14, 0);
    stdscrAddstr( "enter local average (1-255) or \"just\" <enter> to cancel: ");
		SetWin32CursorOn();   		
		gets_s(strLocalAvg);
		if(strlen(strLocalAvg) == 0)
		{
			stdscrMove(scHeight - 14, 0); 
			stdscrErase(SCRN_WIDTH); 
			SetWin32CursorOff();   		
			system("cls");
			return FALSE;
		}
		if((atoi(strLocalAvg) < 1) || (atoi(strLocalAvg) > 255))
		{
			stdscrMove(scHeight - 14, 0); stdscrErase(SCRN_WIDTH);
			stdscrRefresh(); stdscrMove(scHeight - 14, 0);
			printf( "you entered invalid value %u - any key to continue: ", atoi(strLocalAvg));
			getch();
			continue;
		}
		break;
	}
	bLocAvg = atoi(strLocalAvg);
	SetWin32CursorOff();
	system("cls");
  return TRUE;
}

double doAvg(double dataPnt, BYTE x)  //datapoint for chan x passed in
{
  if(uiScanNumber == 1)
	{
    for(BYTE n = 0; n < bLocAvg; n++)            //preload average array to first data value
      dblAvgArray [(n * 8) + x] = dataPnt;
    dblAvgTot[x] = (dataPnt * bLocAvg);        //preload total of those data points
	}
  else
	{			//(scanNumber MOD LocAvg) is pointer into array  
    dblAvgTot[x] = dblAvgTot[x] + dataPnt - dblAvgArray[((uiScanNumber % bLocAvg)* 8) + x];
    dblAvgArray[((uiScanNumber % bLocAvg)* 8) + x] = dataPnt;			//add new, and subtract old data
    dataPnt= dblAvgTot[x] / bLocAvg;															//then update array value, and return new average
	}
  return double(dataPnt);
}




BOOLEAN scanFunc(void)
{
  UINT uiScanNumber = 0;
	UINT uiDataPnt = 0;
  BYTE bErrorType = 0;
  BYTE bGotDevError = 0;
  BOOLEAN fStillScanning = FALSE;
  UINT uiStatusCode = 0;
	memset(dblAvgArray, 0, sizeof(dblAvgArray));
	memset(dblAvgTot, 0, sizeof(dblAvgTot));

  if(fDoLog)createScanLog();           // open log file
     
	#define top 1
	system("cls");
  stdscrMove(top, 1);
	printf("E)End scan    O)digital Output    A)Analog output");
  if(fSingle)printf("    C)Change channel");
  if(fDoLog)printf("    logging to file:  \"%s\")", logFileName);
  if(!fDoScreen)printf("    no screen display of data)");
  stdscrRefresh();
	if(!EX_SetScanType(usActiveDevID, bScanType))
	{
    stdscrMove(top + 1, 0);
    stdscrAddstr ("failed scan type set. <ENTER> to exit scan: "); stdscrRefresh();
		SetWin32CursorOn(); getch(); SetWin32CursorOff(); system("cls");
		if(hScanLogFile){ CloseHandle(hScanLogFile); hScanLogFile = NULL; }
		return FALSE;
	}

	#define SCAN_USE_DATA_LOG_FILE  0x1
	#define SCAN_USE_DATA_ARRAY  0x2
	#define SCAN_STOP_ON_DATA_ARRAY_WRAP  0x4
	#define SCAN_LIMIT_LOG_FILE_SIZE  0x8
  UINT uiDataLogMethod = SCAN_USE_DATA_ARRAY;  //to log to file too, add:  " | SCAN_USE_DATA_LOG_FILE"
	if(!EX_SetDataLogOptions(usActiveDevID, uiDataLogMethod, 0, 0))
	{
    stdscrMove(top + 1, 0);
    stdscrAddstr ("failed scan data log options set. <ENTER> to exit scan: "); stdscrRefresh();
		SetWin32CursorOn(); getch(); SetWin32CursorOff(); system("cls");
		if(hScanLogFile){ CloseHandle(hScanLogFile); hScanLogFile = NULL; }
		return FALSE;
	}
	if(!EX_SetMultiChanScanChans(usActiveDevID, bNumChans))
	{
    stdscrMove(top + 1, 0);
    stdscrAddstr ("failed scan data log options set. <ENTER> to exit scan: "); stdscrRefresh();
		SetWin32CursorOn(); getch(); SetWin32CursorOff(); system("cls");
		if(hScanLogFile){ CloseHandle(hScanLogFile); hScanLogFile = NULL; }
		return FALSE;
	}
  if(!EX_Run(usActiveDevID))
	{
    stdscrMove(top + 1, 0);
    stdscrAddstr ("failed EX_Run. <ENTER> to exit scan: ");
		SetWin32CursorOn(); getch(); SetWin32CursorOff(); system("cls");
		if(hScanLogFile) { CloseHandle(hScanLogFile); hScanLogFile = NULL; }
		system("cls");
		return FALSE;
	}
	fIsScanning = TRUE;
	char strStartTime [128]; 
	time_t now = time(0);
	tm *ltm = localtime(&now);
	sprintf(strStartTime,  "%u-%u-%u_%u-%02u-%u", 1900+ltm->tm_year, 1 + ltm->tm_mon, ltm->tm_mday, ltm->tm_hour, 1 + ltm->tm_min,1 + ltm->tm_sec);
	UINT uiSTime = clock();
	double pdblDataBuff[1600];
	BYTE pbDiginBuff[1600];
	UINT uiNumScansToReturn = 0;					//maximun number of scans to collect in one call to GetScanData
	BOOLEAN fGetAllRequested = FALSE;			//return with however many scans are ready, up to maximum
	UINT puiNumScansReturned = 0;
	UINT puiTotalBuffSize = 400;
	UINT puiNumScansRemainingInBuff = 0;	//should be 0 if you are keeping up with the data
  stdscrMove(top + 1, 0);
	printf(" Doing %s:      Start Time: %s", stypestr[bScanType], strStartTime);
  stdscrMove(top + 2, 0); stdscrAddstr("   scan#   ");
  if(fDoLog) fprintf(fpScanLogFile,  "    scan#  "); 
  if(fSingle)
	{
    printf(" Chan %d    ", bCurChan);
		if(fDig)stdscrAddstr(" DI ");
    if(fDoLog)
		{
      fprintf(fpScanLogFile,  " Chan %d", bCurChan);   
      if(fDig)fprintf(fpScanLogFile,  " DI ");
		}
	}
  else
	{
    for(BYTE x = 0; x < bNumChans; x++)
		{
			printf("  Chan %d  ", x);
			if(fDoLog)fprintf(fpScanLogFile,  "   Chan %d  ", x);
      if(fDig)
			{
        stdscrAddstr("  DI ");
        if(fDoLog) fprintf(fpScanLogFile,  " DI "); 
			}
		}
	}

  if(fCal)
	{
    stdscrAddstr("  offset  ");
    if(fDoLog) fprintf(fpScanLogFile,  "   offset  "); 
    if(fDig)
		{
      stdscrAddstr("  DI ");
      if(fDoLog)fprintf(fpScanLogFile,  " DI ");
		}
	}
  stdscrAddstr("  buff  ");
  if(fDoLog) fprintf(fpScanLogFile,  "\n"); fflush(fpScanLogFile);
  BYTE bCurRow = top + 4;
  BYTE endScan = 0;
  BYTE bRetVal = 0;
  BYTE bIsScanning = 1;				//flag used mostly for screen locations
	BOOLEAN pfStillScanning = FALSE;
	UINT puiStatusCode = 0;
	int keyhit = ' ';
  while (TRUE)
	{
		keyhit = ' ';
		if(kbhit()) keyhit = toupper((char)getch());

    if(keyhit == 'E')
		{
      //stdscrMove(scHeight - 2, 0);; stdscrErase(SCRN_WIDTH); stdscrRefresh(); stdscrMove(scHeight - 2, 0);;
      stdscrMove(scHeight - 2, 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); stdscrMove(scHeight - 2, 0);

      stdscrAddstr ("ending scan - wait . . . ");
      break;
		}
    if(keyhit == 'C' && fSingle) doSendChan();
    if(keyhit == 'O') doDigout();
    if (keyhit == 'A') doAnalogOut();
    if(EX_CheckScanStatus(usActiveDevID, &puiNumScansRemainingInBuff, &pfStillScanning, &puiStatusCode))
		{
      if(pfStillScanning == 0)
			{
				stdscrMove(scHeight - 2, 0);; stdscrErase(SCRN_WIDTH); stdscrRefresh(); stdscrMove(scHeight - 2, 0);;
				stdscrAddstr ("  scan stopped");
        break;											//still need to stop/complete scan
			}
		}
    else
		{
      stdscrMove((scHeight - 4), 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); 
      stdscrMove((scHeight - 4), 0); stdscrAddstr("failed EX_CheckScanStatus - may have lost connection or power - return to main menu");
      stdscrRefresh();
      bGotDevError = 1;

//3-26-18 test scan restart ######################
//3-27-18 commented following lines
   //   stdscrMove((scHeight - 4), 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); 
   //   stdscrMove((scHeight - 4), 0); stdscrAddstr("any key to reinit/restart-scan");
			//getch();		
			//if(EX_Stop(usActiveDevID))
			//{
			//	stdscrMove((scHeight - 4), 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); stdscrMove((scHeight - 4), 0);
			//	stdscrAddstr ("  EX_Stop success");
			//	stdscrMove((scHeight - 4), 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); stdscrMove((scHeight - 4), 0);
			//	stdscrAddstr ("calling EX_StopComplete . . .");
			//	//sprintf(strEndTime,  "scanLog\\%u-%u-%u_%u-%02u-%u.txt", 1900+ltm->tm_year, 1 + ltm->tm_mon, ltm->tm_mday, ltm->tm_hour, 1 + ltm->tm_min,1 + ltm->tm_sec);
			//	if(EX_StopComplete(usActiveDevID))
			//	{
			//		stdscrMove((scHeight - 4), 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); stdscrMove((scHeight - 4), 0);
			//		stdscrAddstr ("  EX_StopComplete success");
			//	}
			//}

			if(fDoLog)
			{
				fclose(fpScanLogFile);
				fpScanLogFile = NULL;
			}

//3-27-18 ########################################################
			stdscrMove((scHeight - 4), 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); 
			stdscrMove((scHeight - 4), 0); stdscrAddstr("scan error - 'S' to attempt to restart, 'E' to end scan");
			int keyhit2 = toupper((char)getch());
			if (keyhit2 == 'S')
			{
				fGotDevError_global = true;
				return TRUE;
			}
//################################################################

//3-27-18d
initDone = false;
return true;


      break;						//still need to stop/complete scan
		}

    if(fSingle) uiNumScansToReturn = 50;			//optimize these two values for your system and application
    else uiNumScansToReturn = 8;							//getting more scans at one time is more efficient, but can increase latency

    if (puiNumScansRemainingInBuff >= 1)			//at least one scan is ready to read
		{
      if(fDig) bRetVal = EX_GetScanDataDblDigin(usActiveDevID, pdblDataBuff, pbDiginBuff, uiNumScansToReturn, fGetAllRequested, 
                      &puiNumScansReturned, &puiNumScansRemainingInBuff, &pfStillScanning, &puiTotalBuffSize, &puiStatusCode);
      else bRetVal = EX_GetScanDataDbl(usActiveDevID, pdblDataBuff, uiNumScansToReturn, fGetAllRequested, 
                      &puiNumScansReturned, &puiNumScansRemainingInBuff, &pfStillScanning, &puiTotalBuffSize, &puiStatusCode);
      if(bRetVal)
			{
				//This routine is used for all scanning. It accepts input of one or more scans
				//as specified by puiNumScansReturned. Data is removed from the buffer and displayed on the screen
				//and/or sent to a data file based on the flags fDoScreen and fDoLog. Flags fCal and fDig are set to
				//match the scan type so that offset and digital input data are handled appropriately. Function doAvg
				//performs running average on LocAvg scans
				for(UINT n = 0; n < puiNumScansReturned; n++)					//loop through for all scans
				{
          if(fDoScreen)
					{
            stdscrMove(bCurRow, 0);														//for screen display only 
            printf("% 8u    ", uiScanNumber);									//display scan number on screen
					}
          if(fDoLog) 
						fprintf(fpScanLogFile,  "%8u    ", uiScanNumber);	//write scan number to file
          uiScanNumber++;   
          UINT uiBase = int(n * (bNumChans + fCal));					//pointer into scan array 
					DOUBLE dblOffset = 0;
          if(fCal) dblOffset = double(pdblDataBuff[uiBase + bNumChans]);	//get dblOffset, if cal scan
          for(BYTE x = 0; x < bNumChans; x++)									//loop through all scan channels
					{
            double dblDataPnt = double(pdblDataBuff[uiBase + x]);	//get datapoint
            if(fCal) dblDataPnt = dblDataPnt - dblOffset;			//subtract dblOffset, if cal scan
            dblDataPnt = doAvg(dblDataPnt,x);									//do runningaverage, initialize if first time, only  #telM
            if(fDig) bDigin = pbDiginBuff[uiBase + x];				//get DigIn, if active 
            if(fDoScreen)
						{
							printf ("% 2.6lf  ", dblDataPnt);	
							if(fDig) printf ("%2d  ", bDigin);
						}
            if(fDoLog)
						{
              if(dblDataPnt < 0)fprintf(fpScanLogFile,  "%1.6lf  ", double(dblDataPnt));	//write results to file 
							else fprintf(fpScanLogFile,  "% 2.6lf  ", double(dblDataPnt));
              if(fDig)fprintf(fpScanLogFile,  "%2d  ", bDigin);
						}
					}
          if(fDoScreen)																					//done with all channels except dblOffset
					{
            if(fCal)
						{
              if(dblOffset < 0)printf("%1.6lf  ", dblOffset);		//if calscan print dblOffset and DigIn, 
							else printf("% 2.6lf  ", dblOffset);
              if(fDig)printf("%2d  ", pbDiginBuff[uiBase + bNumChans]);	//if active
						}
            printf ("% 5d", puiNumScansRemainingInBuff);				//just to warn if falling behind the data 
            stdscrRefresh();  
            bCurRow++;                                          //housekeeping for screen     
            //3-27-18e if(bCurRow > 47) bCurRow = (top + 4);
            if(bCurRow > (scHeight - 4)) bCurRow = (top + 4);

					}
          if(fDoLog)																						//finish writing scan to log file
					{
            if(fCal)
						{
              if(dblOffset < 0)fprintf(fpScanLogFile,  "%1.6lf  ", dblOffset);			//if calscan write dblOffset and DigIn,
							else fprintf(fpScanLogFile,  "% 2.6lf  ", dblOffset);
              if(fDig)fprintf(fpScanLogFile,  "%2d  ", (pbDiginBuff[uiBase + bNumChans]));	//if active
						}
            fprintf(fpScanLogFile,  "\n");											//new line in file
					}
				}
        stdscrRefresh();
			}
		}
	}
  //end of scanloop

  stdscrMove((scHeight - 4), 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); stdscrMove((scHeight - 4), 0);
  stdscrAddstr ("calling EX_Stop . . .");
	now = time(0);
	ltm = localtime(&now);

	char strEndTime[128];
	sprintf(strEndTime,  "scanLog\\%u-%u-%u_%u-%02u-%u.txt", 1900+ltm->tm_year, 1 + ltm->tm_mon, ltm->tm_mday, ltm->tm_hour, 1 + ltm->tm_min,1 + ltm->tm_sec);
	clock_t uiETime = clock();

  if(EX_Stop(usActiveDevID))
	{
    stdscrMove((scHeight - 4), 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); stdscrMove((scHeight - 4), 0);
    stdscrAddstr ("  EX_Stop success");
    stdscrMove((scHeight - 4), 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); stdscrMove((scHeight - 4), 0);
    stdscrAddstr ("calling EX_StopComplete . . .");
		sprintf(strEndTime,  "scanLog\\%u-%u-%u_%u-%02u-%u.txt", 1900+ltm->tm_year, 1 + ltm->tm_mon, ltm->tm_mday, ltm->tm_hour, 1 + ltm->tm_min,1 + ltm->tm_sec);
    if(EX_StopComplete(usActiveDevID))
		{
      stdscrMove((scHeight - 4), 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); stdscrMove((scHeight - 4), 0);
      stdscrAddstr ("  EX_StopComplete success");
		}
	}
  else
	{
    stdscrMove((scHeight - 4), 0); stdscrErase(SCRN_WIDTH); stdscrRefresh(); stdscrMove((scHeight - 4), 0);
    stdscrAddstr ("  Failed EX_Stop");
    bIsScanning = 0;
	}

  if(fDoLog)
	{
    fprintf(fpScanLogFile,  "\nAveraging: %d", bLocAvg);
    fprintf(fpScanLogFile,  "\nInternal Data Rate: %.2f", dblActualRate);
    fprintf(fpScanLogFile,  "\nScan Rate: %f", (dblActualRate/((bNumChans + fCal) * 5)));
    fprintf(fpScanLogFile,  "\nStarted: %s", strStartTime);
    fprintf(fpScanLogFile,  "\nFinished: %s", strEndTime);
    fprintf(fpScanLogFile,  "\nElapsed: %u  seconds", (uiETime - uiSTime));
    fclose(fpScanLogFile);
		fpScanLogFile = NULL;
	}

  if(fSingle) bNumChans = bNumChansTemp;    //tel restore channel count for multichannel scans
	return TRUE;
}


#include <direct.h>
BOOLEAN createScanLog(void)
{
	time_t now = time(0);
	tm *ltm = localtime(&now);
	mkdir("scanLog");		//does nothing if already exists
	sprintf(logFileName,  "scanLog\\%u-%u-%u_%u-%02u-%u.txt", 1900+ltm->tm_year, 1 + ltm->tm_mon, ltm->tm_mday, ltm->tm_hour, 1 + ltm->tm_min,1 + ltm->tm_sec);
	fpScanLogFile = fopen(logFileName, "w+");
	return TRUE;
}












